<?php

namespace Drupal\entity_notify\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form that configures forms module settings.
 */
class EntityNotifySettingsForm extends ConfigFormBase {

  /**
   * The Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_notify_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['entity_notify.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('entity_notify.settings');

    $all_entity_types = $this->entityTypeManager->getDefinitions();
    $content_entity_types = [];
    /** @var \Drupal\Core\Entity\EntityTypeInterface[] $entity_type_options */
    foreach ($all_entity_types as $entity_type) {
      if (($entity_type instanceof ContentEntityTypeInterface)) {
        $content_entity_types[$entity_type->id()] = $entity_type->getLabel();
      }
    }

    unset($content_entity_types['file']);
    unset($content_entity_types['user']);
    unset($content_entity_types['menu_link_content']);
    unset($content_entity_types['path_alias']);
    unset($content_entity_types['shortcut']);

    $form['enabled_target_entity_types'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Enabled target entity types'),
      '#tree' => TRUE,
    ];
    $form['enabled_target_entity_types']['entity_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Target entity types'),
      '#options' => $content_entity_types,
      '#default_value' => $config->get('enabled_target_entity_types') ?: array_keys($content_entity_types),
      '#required' => TRUE,
    ];

    $roles = $config->get('entity_notify_roles');

    $form['fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Send mail to'),
    ];
    $form['fieldset']['entity_notify_admin'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('admin'),
      '#default_value' => $config->get('entity_notify_admin'),
      '#description' => $this->t('Send mail to user with uid=1'),
    ];
    $form['fieldset']['entity_notify_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#default_value' => !empty($roles) ? $roles : [],
      '#options' => array_map('\Drupal\Component\Utility\Html::escape', user_role_names()),
      '#description' => $this->t('Send mail to users with selected roles'),
    ];
    $form['fieldset']['entity_notify_maillist'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom mail list'),
      '#default_value' => $config->get('entity_notify_maillist'),
      '#description' => $this->t('Send mail to non-registered users. Delimiter: comma'),
    ];
    $form['fieldset']['entity_notify_telegram'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Send message to Telegram users'),
      '#default_value' => $config->get('entity_notify_telegram'),
    ];
    $form['fieldset']['entity_notify_telegram_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Telegram Settings'),
      '#states' => [
        'visible' => [
          '#edit-entity-notify-telegram' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['fieldset']['entity_notify_telegram_fieldset']['entity_notify_telegram_bottoken'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Bot token'),
      '#default_value' => $config->get('entity_notify_telegram_bottoken'),
    ];
    $form['fieldset']['entity_notify_telegram_fieldset']['entity_notify_telegram_chatids'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Chat IDs'),
      '#default_value' => $config->get('entity_notify_telegram_chatids'),
      '#description' => $this->t('Delimiter: comma'),
    ];
    $form['fieldset']['entity_notify_telegram_fieldset']['entity_notify_telegram_proxy'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use proxy'),
      '#default_value' => $config->get('entity_notify_telegram_proxy'),
    ];
    $form['fieldset']['entity_notify_telegram_fieldset']['entity_notify_telegram_proxy_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Proxy Settings (SOCKS 5)'),
      '#states' => [
        'visible' => [
          '#edit-entity_notify-telegram-proxy' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['fieldset']['entity_notify_telegram_fieldset']['entity_notify_telegram_proxy_fieldset']['entity_notify_telegram_proxy_server'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Server name or IP address'),
      '#default_value' => $config->get('entity_notify_telegram_proxy_server'),
      '#description' => $this->t('For example: 127.0.0.1:1234'),
    ];
    $form['fieldset']['entity_notify_telegram_fieldset']['entity_notify_telegram_proxy_fieldset']['entity_notify_telegram_proxy_login'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Server login'),
      '#default_value' => $config->get('entity_notify_telegram_proxy_login'),
    ];
    $form['fieldset']['entity_notify_telegram_fieldset']['entity_notify_telegram_proxy_fieldset']['entity_notify_telegram_proxy_password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Server password'),
      '#default_value' => $config->get('entity_notify_telegram_proxy_password'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('entity_notify.settings')
      ->set('enabled_target_entity_types', array_values(array_filter($form_state->getValue('enabled_target_entity_types')['entity_types'])))
      ->set('entity_notify_admin', $form_state->getValue('entity_notify_admin'))
      ->set('entity_notify_roles', $form_state->getValue('entity_notify_roles'))
      ->set('entity_notify_maillist', $form_state->getValue('entity_notify_maillist'))
      ->set('entity_notify_telegram', $form_state->getValue('entity_notify_telegram'))
      ->set('entity_notify_telegram_bottoken', $form_state->getValue('entity_notify_telegram_bottoken'))
      ->set('entity_notify_telegram_chatids', $form_state->getValue('entity_notify_telegram_chatids'))
      ->set('entity_notify_telegram_proxy', $form_state->getValue('entity_notify_telegram_proxy'))
      ->set('entity_notify_telegram_proxy_server', $form_state->getValue('entity_notify_telegram_proxy_server'))
      ->set('entity_notify_telegram_proxy_login', $form_state->getValue('entity_notify_telegram_proxy_login'))
      ->set('entity_notify_telegram_proxy_password', $form_state->getValue('entity_notify_telegram_proxy_password'))
      ->save();
    $this->messenger()->addMessage($this->t('The configuration options have been saved.'));
  }

}
